from plone.rest.interfaces import IShouldAllowAcquiredItemPublication
from plone.rest.traverse import RESTWrapper
from zope.component import adapter


@adapter(RESTWrapper)
def rest_allowed(wrapper):
    return IShouldAllowAcquiredItemPublication(wrapper.context)



from base64 import b64encode
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from zExceptions import NotFound
from zope.event import notify
from ZPublisher.pubevents import PubAfterTraversal
from ZPublisher.pubevents import PubStart

import unittest


try:
    from Products.CMFCore.interfaces import IShouldAllowAcquiredItemPublication

    IShouldAllowAcquiredItemPublication  # flake8

    HAS_CMFCORE_32 = True
except ImportError:
    HAS_CMFCORE_32 = False


@unittest.skipUnless(
    not HAS_CMFCORE_32,
    "Older Plone versions don't have CMFCore>=3.2",
)
class TestExplicitAcquisitionUnavailable(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Document", id="foo")

    def traverse(self, path="/plone", accept="application/json", method="GET"):
        request = self.layer["request"]
        request.environ["PATH_INFO"] = path
        request.environ["PATH_TRANSLATED"] = path
        request.environ["HTTP_ACCEPT"] = accept
        request.environ["REQUEST_METHOD"] = method
        auth = f"{SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}"
        b64auth = b64encode(auth.encode("utf8"))
        request._auth = "Basic %s" % b64auth.decode("utf8")
        notify(PubStart(request))
        return request.traverse(path)

    def test_portal_root(self):
        self.traverse("/plone")
        notify(PubAfterTraversal(self.request))

    def test_portal_foo(self):
        self.traverse("/plone/foo")
        notify(PubAfterTraversal(self.request))

    def test_portal_foo_acquired(self):
        self.traverse("/plone/foo/foo")
        notify(PubAfterTraversal(self.request))


@unittest.skipUnless(
    HAS_CMFCORE_32,
    "We have Products.CMFCore >= 3.2",
)
class TestExplicitAcquisitionAvailable(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        import Products.CMFCore.explicitacquisition

        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Document", id="foo")
        self.PREVIOUS_SKIP_PTA = Products.CMFCore.explicitacquisition.SKIP_PTA

    def tearDown(self):
        import Products.CMFCore.explicitacquisition

        Products.CMFCore.explicitacquisition.SKIP_PTA = self.PREVIOUS_SKIP_PTA

    def traverse(self, path="/plone", accept="application/json", method="GET"):
        request = self.layer["request"]
        request.environ["PATH_INFO"] = path
        request.environ["PATH_TRANSLATED"] = path
        request.environ["HTTP_ACCEPT"] = accept
        request.environ["REQUEST_METHOD"] = method
        auth = f"{SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}"
        b64auth = b64encode(auth.encode("utf8"))
        request._auth = "Basic %s" % b64auth.decode("utf8")
        notify(PubStart(request))
        return request.traverse(path)

    def test_portal_root(self):
        import Products.CMFCore.explicitacquisition

        self.traverse("/plone")
        Products.CMFCore.explicitacquisition.SKIP_PTA = True
        notify(PubAfterTraversal(self.request))

        Products.CMFCore.explicitacquisition.SKIP_PTA = False
        notify(PubAfterTraversal(self.request))

    def test_portal_foo(self):
        import Products.CMFCore.explicitacquisition

        self.traverse("/plone/foo")
        Products.CMFCore.explicitacquisition.SKIP_PTA = True
        notify(PubAfterTraversal(self.request))

        Products.CMFCore.explicitacquisition.SKIP_PTA = False
        notify(PubAfterTraversal(self.request))

    def test_portal_foo_acquired(self):
        import Products.CMFCore.explicitacquisition

        self.traverse("/plone/foo/foo")

        Products.CMFCore.explicitacquisition.SKIP_PTA = True
        notify(PubAfterTraversal(self.request))

        Products.CMFCore.explicitacquisition.SKIP_PTA = False
        with self.assertRaises(NotFound):
            notify(PubAfterTraversal(self.request))



from zope.interface import Interface


class IAPIRequest(Interface):
    """Marker for API requests."""


class IService(Interface):
    """Marker for REST services."""


class ICORSPolicy(Interface):
    """Provides methods for processing simple and preflight CORS requests by
    adding access control headers.
    """

    def process_simple_request():
        """Process a simple request"""

    def process_preflight_request():
        """Process a preflight request"""


try:
    from Products.CMFCore.interfaces import IShouldAllowAcquiredItemPublication
except ImportError:

    class IShouldAllowAcquiredItemPublication(Interface):
        pass



from base64 import b64encode
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.base.interfaces import INavigationRoot
from plone.rest.service import Service
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from Products.SiteAccess.VirtualHostMonster import VirtualHostMonster
from zExceptions import NotFound
from zExceptions import Redirect
from zope.event import notify
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IBrowserView
from ZPublisher import BeforeTraverse
from ZPublisher.pubevents import PubStart

import unittest


class TestTraversal(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def traverse(self, path="/plone", accept="application/json", method="GET"):
        request = self.layer["request"]
        request.environ["PATH_INFO"] = path
        request.environ["PATH_TRANSLATED"] = path
        request.environ["HTTP_ACCEPT"] = accept
        request.environ["REQUEST_METHOD"] = method
        auth = f"{SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}"
        b64auth = b64encode(auth.encode("utf8"))
        request._auth = "Basic %s" % b64auth.decode("utf8")
        notify(PubStart(request))
        return request.traverse(path)

    def test_json_request_on_portal_root_returns_service(self):
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_json_request_on_portal_root_with_layout_returns_service(self):
        self.portal.setLayout("summary_view")
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_json_request_on_portal_root_with_default_page_returns_service(self):
        self.portal.invokeFactory("Document", id="doc1")
        self.portal.setDefaultPage("doc1")
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_json_request_on_content_object_returns_service(self):
        self.portal.invokeFactory("Document", id="doc1")
        obj = self.traverse(path="/plone/doc1")
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_html_request_on_portal_root_returns_default_view(self):
        obj = self.traverse(accept="text/html")
        self.assertEqual(self.portal.getDefaultLayout(), obj.__name__)

    def test_html_request_on_portal_root_returns_dynamic_view(self):
        self.portal.setLayout("summary_view")
        obj = self.traverse(accept="text/html")
        self.assertEqual("summary_view", obj.__name__)

    def test_html_request_on_portal_root_returns_default_page(self):
        self.portal.invokeFactory("Document", id="doc1")
        self.portal.setDefaultPage("doc1")
        obj = self.traverse(accept="text/html")
        self.assertEqual("document_view", obj.__name__)

    def test_json_request_on_object_with_multihook(self):
        doc1 = self.portal[self.portal.invokeFactory("Document", id="doc1")]

        # Register a function to be called before traversal
        def btr_test(container, request):
            request._btr_test_called = 1

        doc1.btr_test = btr_test
        nc = BeforeTraverse.NameCaller("btr_test")
        BeforeTraverse.registerBeforeTraverse(doc1, nc, "Document/btr_test")

        obj = self.traverse(path="/plone/doc1")
        self.assertTrue(isinstance(obj, Service), "Not a service")
        self.assertEqual(1, self.request._btr_test_called)

    def test_json_request_on_existing_view_returns_named_service(self):
        obj = self.traverse("/plone/search")
        self.assertTrue(isinstance(obj, Service), "Not a service")

        folder = self.portal[self.portal.invokeFactory("Folder", id="folder1")]
        alsoProvides(folder, INavigationRoot)
        obj = self.traverse("/plone/folder1/search")
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_html_request_on_existing_view_returns_view(self):
        obj = self.traverse(path="/plone/search", accept="text/html")
        self.assertFalse(isinstance(obj, Service), "Got a service")

        folder = self.portal[self.portal.invokeFactory("Folder", id="folder1")]
        alsoProvides(folder, INavigationRoot)
        obj = self.traverse(path="/plone/folder1/search", accept="text/html")
        self.assertFalse(isinstance(obj, Service), "Got a service")

    def test_html_request_via_api_returns_service(self):
        obj = self.traverse(path="/plone/++api++", accept="text/html")
        self.assertTrue(isinstance(obj, Service), "Not a service")

    def test_html_request_via_double_apis_raises_redirect(self):
        portal_url = self.portal.absolute_url()
        with self.assertRaises(Redirect) as exc:
            self.traverse(path="/plone/++api++/++api++", accept="text/html")
        self.assertEqual(
            exc.exception.headers["Location"],
            f"{portal_url}/++api++",
        )

    def test_html_request_via_multiple_apis_raises_redirect(self):
        portal_url = self.portal.absolute_url()
        with self.assertRaises(Redirect) as exc:
            self.traverse(
                path="/plone/++api++/++api++/++api++/search", accept="text/html"
            )
        self.assertEqual(
            exc.exception.headers["Location"],
            f"{portal_url}/++api++/search",
        )

    def test_html_request_via_multiple_bad_apis_raises_not_found(self):
        with self.assertRaises(NotFound):
            self.traverse(path="/plone/++api++/search/++api++", accept="text/html")

    def test_virtual_hosting(self):
        app = self.layer["app"]
        vhm = VirtualHostMonster()
        vhm.id = "virtual_hosting"
        vhm.addToContainer(app)
        obj = self.traverse(
            path="/VirtualHostBase/http/localhost:8080/plone/VirtualHostRoot/"
        )  # noqa
        self.assertTrue(isinstance(obj, Service), "Not a service")
        del app["virtual_hosting"]

    def test_json_request_to_regular_view_returns_view(self):
        obj = self.traverse("/plone/folder_contents")
        self.assertTrue(IBrowserView.providedBy(obj), "IBrowserView expected")

        self.portal[self.portal.invokeFactory("Folder", id="folder1")]
        obj = self.traverse("/plone/folder1/folder_contents")
        self.assertTrue(IBrowserView.providedBy(obj), "IBrowserView expected")

    def test_json_request_to_view_namespace_returns_view(self):
        obj = self.traverse("/plone/@@folder_contents")
        self.assertTrue(IBrowserView.providedBy(obj), "IBrowserView expected")

        self.portal[self.portal.invokeFactory("Folder", id="folder1")]
        obj = self.traverse("/plone/folder1/@@folder_contents")
        self.assertTrue(IBrowserView.providedBy(obj), "IBrowserView expected")



from plone.rest import Service

import json


class BaseService(Service):
    def render(self):
        self.request.response.setHeader("Content-Type", "application/json")
        return json.dumps(self.data(), indent=2, sort_keys=True)


class Get(BaseService):
    def data(self):
        return {"method": "GET", "id": self.context.id}


class Post(BaseService):
    def data(self):
        return {"method": "POST", "id": self.context.id}


class Put(BaseService):
    def data(self):
        return {"method": "PUT", "id": self.context.id}


class Delete(BaseService):
    def data(self):
        return {"method": "DELETE", "id": self.context.id}


class Patch(BaseService):
    def data(self):
        return {"method": "PATCH", "id": self.context.id}


class Options(BaseService):
    def data(self):
        return {"method": "OPTIONS", "id": self.context.id}


class NamedGet(BaseService):
    def data(self):
        return {"service": "named get"}


class NamedPost(BaseService):
    def data(self):
        return {"service": "named post"}


class NamedPut(BaseService):
    def data(self):
        return {"service": "named put"}


class NamedDelete(BaseService):
    def data(self):
        return {"service": "named delete"}


class NamedPatch(BaseService):
    def data(self):
        return {"service": "named patch"}


class NamedOptions(BaseService):
    def data(self):
        return {"service": "named options"}



from BTrees.OOBTree import OOSet
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.errors import ErrorHandling
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from zope.component import queryUtility

import requests
import transaction
import unittest


class TestRedirects(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.portal.invokeFactory("Folder", id="folder-old")
        self.portal.manage_renameObject("folder-old", "folder-new")
        transaction.commit()

    def test_get_to_moved_item_causes_302_redirect(self):
        response = requests.get(
            self.portal_url + "/folder-old",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(self.portal_url + "/folder-new", response.headers["Location"])
        self.assertEqual(b"", response.raw.read())

    def test_get_to_moved_item_causes_302_redirect_with_api_traverser(self):
        response = requests.get(
            self.portal_url + "/++api++/folder-old",
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/++api++/folder-new", response.headers["Location"]
        )
        self.assertEqual(b"", response.raw.read())
        # follow the new location
        response = requests.get(
            response.headers["Location"],
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("application/json", response.headers["Content-type"])
        self.assertEqual({"id": "folder-new", "method": "GET"}, response.json())

    def test_get_to_moved_item_causes_302_redirect_with_rest_view(self):
        response = requests.get(
            self.portal_url + "/++api++/folder-old/@actions",
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/++api++/folder-new/@actions",
            response.headers["Location"],
        )
        self.assertEqual(b"", response.raw.read())

    def test_post_to_moved_item_causes_307_redirect(self):
        response = requests.post(
            self.portal_url + "/folder-old",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(307, response.status_code)
        self.assertEqual(self.portal_url + "/folder-new", response.headers["Location"])
        self.assertEqual(b"", response.raw.read())

    def test_post_to_moved_item_causes_307_redirect_with_api_traverser(self):
        response = requests.post(
            self.portal_url + "/++api++/folder-old",
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(307, response.status_code)
        self.assertEqual(
            self.portal_url + "/++api++/folder-new", response.headers["Location"]
        )
        self.assertEqual(b"", response.raw.read())

    def test_unauthorized_request_to_item_still_redirects_first(self):
        response = requests.get(
            self.portal_url + "/folder-old",
            headers={"Accept": "application/json"},
            # No auth
            allow_redirects=False,
        )

        # A request to the old URL of an item where the user doesn't have
        # necessary permissions will still result in a redirect
        self.assertEqual(302, response.status_code)
        self.assertEqual(self.portal_url + "/folder-new", response.headers["Location"])
        self.assertEqual(b"", response.raw.read())

        # Following the redirect then leads to an item that will produce a 401
        response = requests.get(
            response.headers["Location"],
            headers={"Accept": "application/json"},
            # No auth
            allow_redirects=False,
        )
        self.assertEqual(401, response.status_code)

    def test_query_string_gets_preserved(self):
        response = requests.get(
            self.portal_url + "/folder-old?key=value",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/folder-new?key=value", response.headers["Location"]
        )
        self.assertEqual(b"", response.raw.read())

    def test_named_service_on_moved_item_causes_302_redirect(self):
        response = requests.get(
            self.portal_url + "/folder-old/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/folder-new/namedservice", response.headers["Location"]
        )
        self.assertEqual(b"", response.raw.read())

    def test_named_service_plus_path_parameter_works(self):
        response = requests.get(
            self.portal_url + "/folder-old/namedservice/param",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/folder-new/namedservice/param",
            response.headers["Location"],
        )
        self.assertEqual(b"", response.raw.read())

    def test_redirects_for_regular_views_still_work(self):
        response = requests.get(
            self.portal_url + "/folder-old/@@some-view",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/folder-new/@@some-view", response.headers["Location"]
        )
        self.assertEqual(b"", response.raw.read())

    def test_redirects_for_views_plus_params_plus_querystring_works(self):
        response = requests.get(
            self.portal_url + "/folder-old/@@some-view/param?k=v",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(
            self.portal_url + "/folder-new/@@some-view/param?k=v",
            response.headers["Location"],
        )
        self.assertEqual(b"", response.raw.read())

    def test_doesnt_cause_redirect_loop_on_bogus_storage_entries(self):
        storage = queryUtility(IRedirectionStorage)
        storage._paths["/plone/same"] = "/plone/same"
        storage._rpaths["/plone/same"] = OOSet(["/plone/same"])
        transaction.commit()

        response = requests.get(
            self.portal_url + "/same/@@view",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(404, response.status_code)

    def test_handles_redirects_that_include_querystring_in_old_path(self):
        storage = queryUtility(IRedirectionStorage)
        storage.add("/plone/some-item?key=value", "/plone/new-item")
        transaction.commit()

        response = requests.get(
            self.portal_url + "/some-item?key=value",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(302, response.status_code)
        self.assertEqual(self.portal_url + "/new-item", response.headers["Location"])
        self.assertEqual(b"", response.raw.read())

    def test_handles_redirects_that_are_recursive(self):
        storage = queryUtility(IRedirectionStorage)
        storage.add("/plone/folder-new", "/plone/folder-new/archive")
        transaction.commit()
        # Request should return 404
        response = requests.get(
            self.portal_url + "/folder-new/sub_folder/not-found",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(404, response.status_code)

    def test_aborts_redirect_checks_early_for_app_root(self):
        error_view = ErrorHandling(self.portal, self.portal.REQUEST)
        self.assertIsNone(error_view.find_redirect_if_view_or_service([""], None))

    def test_gracefully_deals_with_missing_request_url(self):
        error_view = ErrorHandling(self.portal, self.portal.REQUEST)
        self.portal.REQUEST["ACTUAL_URL"] = None
        self.assertEqual(False, error_view.attempt_redirect())



from plone.rest.negotiation import lookup_service_id
from plone.rest.negotiation import parse_accept_header
from plone.rest.negotiation import register_service

import unittest


class TestAcceptHeaderParser(unittest.TestCase):
    def test_parse_application_json_accept_header(self):
        accept = "application/json"
        expected = [("application", "json")]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_jquery_json_accept_header(self):
        accept = (
            "text/javascript, application/javascript, "
            "application/ecmascript, application/x-ecmascript, "
            "*/*; q=0.01"
        )
        expected = [
            ("text", "javascript"),
            ("application", "javascript"),
            ("application", "ecmascript"),
            ("application", "x-ecmascript"),
            ("*", "*"),
        ]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_firefox_accept_header(self):
        accept = "text/html,application/xhtml+xml,application/xml;q=0.9," "*/*;q=0.8"
        expected = [
            ("text", "html"),
            ("application", "xhtml+xml"),
            ("application", "xml"),
            ("*", "*"),
        ]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_chrome_accept_header(self):
        accept = (
            "text/html,application/xhtml+xml,application/xml;q=0.9,"
            "image/webp,*/*;q=0.8"
        )
        expected = [
            ("text", "html"),
            ("application", "xhtml+xml"),
            ("application", "xml"),
            ("image", "webp"),
            ("*", "*"),
        ]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_all_media_types_accept_header(self):
        self.assertEqual([("*", "*")], parse_accept_header("*/*"))

    def test_parse_invalid_accept_header(self):
        self.assertEqual([], parse_accept_header("invalid"))

    def test_parse_mimetype_with_extra_slash(self):
        self.assertEqual(
            [("application", "x/y")], parse_accept_header("application/x/y")
        )


class TestServiceRegistry(unittest.TestCase):
    def test_register_media_type(self):
        self.assertEqual(
            "GET_application_json_", register_service("GET", ("application", "json"))
        )
        self.assertEqual(
            "GET_application_json_", lookup_service_id("GET", "application/json")
        )

    def test_register_wildcard_subtype(self):
        self.assertEqual("PATCH_text_*_", register_service("PATCH", ("text", "*")))
        self.assertEqual("PATCH_text_*_", lookup_service_id("PATCH", "text/xml"))

    def test_register_wilcard_type(self):
        self.assertEqual("PATCH_*_*_", register_service("PATCH", ("*", "*")))
        self.assertEqual("PATCH_*_*_", lookup_service_id("PATCH", "foo/bar"))

    def test_service_id_for_multiple_media_types_is_none(self):
        register_service("GET", "application/json")
        self.assertEqual(
            None, lookup_service_id("GET", "application/json,application/javascript")
        )

    def test_service_id_for_invalid_media_type_is_none(self):
        self.assertEqual(None, lookup_service_id("GET", "application-json"))

    def test_service_id_for_not_registered_media_type_is_none(self):
        self.assertEqual(None, lookup_service_id("PUT", "text/html"))

    def test_service_id_for_wildcard_type_is_none(self):
        register_service("GET", "application/json")
        self.assertEqual(None, lookup_service_id("GET", "*/*"))

    def test_service_id_for_wildcard_subtype_is_none(self):
        register_service("GET", "text/xml")
        self.assertEqual(None, lookup_service_id("GET", "text/*"))



from datetime import datetime
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import requests
import transaction
import unittest


class TestDexterityServiceEndpoints(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Document", id="doc1")
        self.document = self.portal["doc1"]
        transaction.commit()

    def test_dexterity_document_get(self):
        response = requests.get(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_document_post(self):
        response = requests.post(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("POST", response.json().get("method"))

    def test_dexterity_document_put(self):
        response = requests.put(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("PUT", response.json().get("method"))

    def test_dexterity_document_patch(self):
        response = requests.patch(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("PATCH", response.json().get("method"))

    def test_dexterity_document_delete(self):
        response = requests.delete(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("DELETE", response.json().get("method"))

    def test_dexterity_document_options(self):
        response = requests.options(
            self.document.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual("doc1", response.json().get("id"))
        self.assertEqual("OPTIONS", response.json().get("method"))

    def test_dexterity_folder_get(self):
        self.portal.invokeFactory("Folder", id="folder")
        transaction.commit()

        response = requests.get(
            self.portal.folder.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("folder", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_news_item_get(self):
        self.portal.invokeFactory("News Item", id="newsitem")
        self.portal.newsitem.title = "My News Item"
        self.portal.newsitem.description = "This is a news item"
        self.portal.newsitem.text = RichTextValue(
            "Lorem ipsum", "text/plain", "text/html"
        )
        image_file = os.path.join(os.path.dirname(__file__), "image.png")
        fd = open(image_file, "rb")
        self.portal.newsitem.image = NamedBlobImage(
            data=fd.read(), contentType="image/png", filename="image.png"
        )
        fd.close()
        self.portal.newsitem.image_caption = "This is an image caption."
        import transaction

        transaction.commit()
        response = requests.get(
            self.portal.newsitem.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("newsitem", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_event_get(self):
        self.portal.invokeFactory("Event", id="event")
        self.portal.event.title = "Event"
        self.portal.event.description = "This is an event"
        self.portal.event.start = datetime(2013, 1, 1, 10, 0)
        self.portal.event.end = datetime(2013, 1, 1, 12, 0)
        import transaction

        transaction.commit()
        response = requests.get(
            self.portal.event.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("event", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_link_get(self):
        self.portal.invokeFactory("Link", id="link")
        self.portal.link.title = "My Link"
        self.portal.link.description = "This is a link"
        self.portal.remoteUrl = "http://plone.org"
        import transaction

        transaction.commit()
        response = requests.get(
            self.portal.link.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("link", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_file_get(self):
        self.portal.invokeFactory("File", id="file")
        self.portal.file.title = "My File"
        self.portal.file.description = "This is a file"
        pdf_file = os.path.join(os.path.dirname(__file__), "file.pdf")
        fd = open(pdf_file, "rb")
        self.portal.file.file = NamedBlobFile(
            data=fd.read(), contentType="application/pdf", filename="file.pdf"
        )
        fd.close()
        intids = getUtility(IIntIds)
        file_id = intids.getId(self.portal.file)
        self.portal.file.file = RelationValue(file_id)
        import transaction

        transaction.commit()

        response = requests.get(
            self.portal.file.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("file", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_image_get(self):
        self.portal.invokeFactory("Image", id="image")
        self.portal.image.title = "My Image"
        self.portal.image.description = "This is an image"
        image_file = os.path.join(os.path.dirname(__file__), "image.png")
        fd = open(image_file, "rb")
        self.portal.image.image = NamedBlobImage(
            data=fd.read(), contentType="image/png", filename="image.png"
        )
        fd.close()
        import transaction

        transaction.commit()

        response = requests.get(
            self.portal.image.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("image", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_dexterity_collection_get(self):
        self.portal.invokeFactory("Collection", id="collection")
        self.portal.collection.title = "My Collection"
        self.portal.collection.description = "This is a collection with two documents"
        self.portal.collection.query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.string.is",
                "v": "Document",
            }
        ]
        self.portal.invokeFactory("Document", id="doc2", title="Document 2")
        import transaction

        transaction.commit()
        response = requests.get(
            self.portal.collection.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual("collection", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))



from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from Products.CMFCore.utils import getToolByName

import requests
import transaction
import unittest


CREDS = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
INVALID_CREDS = ("invalid", "password")
NO_CREDS = ()


class DispatchingTestCase(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def validate(self, expectations, follow_redirects=False):
        failures = []
        for expectation in expectations:
            path, method, creds, expected_status = expectation
            url = self.portal_url + path

            response = requests.request(
                method,
                url,
                headers={"Accept": "application/json"},
                auth=creds,
                allow_redirects=follow_redirects,
            )

            if response.status_code != expected_status:
                request_args = (
                    path,
                    method,
                    creds,
                    "(follow_redirects=%s)" % follow_redirects,
                )
                actual_status = response.status_code
                failure = (request_args, expected_status, actual_status)
                failures.append(failure)

        if failures:
            msg = ""
            for request_args, expected_status, actual_status in failures:
                msg += (
                    "\n"
                    "Request:  %s\n"
                    "Expected: %s\n"
                    "Got:      %s\n" % (request_args, expected_status, actual_status)
                )

            self.fail("The following assertions failed:\n%s" % msg)


class TestDispatchingSiteRoot(DispatchingTestCase):
    def test_site_root_with_creds(self):
        expectations = [
            ("/", "GET", CREDS, 200),
            ("/", "POST", CREDS, 200),
            ("/", "PUT", CREDS, 200),
            ("/", "PATCH", CREDS, 200),
            ("/", "DELETE", CREDS, 200),
            ("/", "OPTIONS", CREDS, 200),
        ]
        self.validate(expectations)

    def test_site_root_without_creds(self):
        expectations = [
            ("/", "GET", NO_CREDS, 200),
            ("/", "POST", NO_CREDS, 401),
            ("/", "PUT", NO_CREDS, 401),
            ("/", "PATCH", NO_CREDS, 401),
            ("/", "DELETE", NO_CREDS, 401),
            ("/", "OPTIONS", NO_CREDS, 200),
        ]
        self.validate(expectations)

    def test_site_root_invalid_creds(self):
        expectations = [
            ("/", "GET", INVALID_CREDS, 200),
            ("/", "POST", INVALID_CREDS, 401),
            ("/", "PUT", INVALID_CREDS, 401),
            ("/", "PATCH", INVALID_CREDS, 401),
            ("/", "DELETE", INVALID_CREDS, 401),
            ("/", "OPTIONS", INVALID_CREDS, 200),
        ]
        self.validate(expectations)


class TestDispatchingNonExistentResource(DispatchingTestCase):
    def test_not_found_with_creds(self):
        expectations = [
            ("/not-found", "GET", CREDS, 404),
            ("/not-found", "POST", CREDS, 404),
            ("/not-found", "PUT", CREDS, 404),
            ("/not-found", "PATCH", CREDS, 404),
            ("/not-found", "DELETE", CREDS, 404),
            ("/not-found", "OPTIONS", CREDS, 404),
        ]
        self.validate(expectations)

    def test_not_found_without_creds(self):
        expectations = [
            ("/not-found", "GET", NO_CREDS, 404),
            ("/not-found", "POST", NO_CREDS, 404),
            ("/not-found", "PUT", NO_CREDS, 404),
            ("/not-found", "PATCH", NO_CREDS, 404),
            ("/not-found", "DELETE", NO_CREDS, 404),
            ("/not-found", "OPTIONS", NO_CREDS, 404),
        ]
        self.validate(expectations)

    def test_not_found_invalid_creds(self):
        expectations = [
            ("/not-found", "GET", INVALID_CREDS, 404),
            ("/not-found", "POST", INVALID_CREDS, 404),
            ("/not-found", "PUT", INVALID_CREDS, 404),
            ("/not-found", "PATCH", INVALID_CREDS, 404),
            ("/not-found", "DELETE", INVALID_CREDS, 404),
            ("/not-found", "OPTIONS", INVALID_CREDS, 404),
        ]
        self.validate(expectations)


class TestDispatchingDexterity(DispatchingTestCase):
    def setUp(self):
        super().setUp()
        self.portal.invokeFactory("Folder", id="private")

        self.portal.invokeFactory("Folder", id="public")
        public_folder = self.portal.public
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(public_folder, "publish")

        transaction.commit()

    def test_private_dx_folder_with_creds(self):
        expectations = [
            ("/private", "GET", CREDS, 200),
            ("/private", "POST", CREDS, 200),
            ("/private", "PUT", CREDS, 200),
            ("/private", "PATCH", CREDS, 200),
            ("/private", "DELETE", CREDS, 200),
            ("/private", "OPTIONS", CREDS, 200),
        ]
        self.validate(expectations)

    def test_private_dx_folder_without_creds(self):
        expectations = [
            ("/private", "GET", NO_CREDS, 401),
            ("/private", "POST", NO_CREDS, 401),
            ("/private", "PUT", NO_CREDS, 401),
            ("/private", "PATCH", NO_CREDS, 401),
            ("/private", "DELETE", NO_CREDS, 401),
            ("/private", "OPTIONS", NO_CREDS, 401),
        ]
        self.validate(expectations)

    def test_private_dx_folder_invalid_creds(self):
        expectations = [
            ("/private", "GET", INVALID_CREDS, 401),
            ("/private", "POST", INVALID_CREDS, 401),
            ("/private", "PUT", INVALID_CREDS, 401),
            ("/private", "PATCH", INVALID_CREDS, 401),
            ("/private", "DELETE", INVALID_CREDS, 401),
            ("/private", "OPTIONS", INVALID_CREDS, 401),
        ]
        self.validate(expectations)

    def test_public_dx_folder_with_creds(self):
        expectations = [
            ("/public", "GET", CREDS, 200),
            ("/public", "POST", CREDS, 200),
            ("/public", "PUT", CREDS, 200),
            ("/public", "PATCH", CREDS, 200),
            ("/public", "DELETE", CREDS, 200),
            ("/public", "OPTIONS", CREDS, 200),
        ]
        self.validate(expectations)

    def test_public_dx_folder_without_creds(self):
        expectations = [
            ("/public", "GET", NO_CREDS, 200),
            ("/public", "POST", NO_CREDS, 401),
            ("/public", "PUT", NO_CREDS, 401),
            ("/public", "PATCH", NO_CREDS, 401),
            ("/public", "DELETE", NO_CREDS, 401),
            ("/public", "OPTIONS", NO_CREDS, 200),
        ]
        self.validate(expectations)

    def test_public_dx_folder_invalid_creds(self):
        expectations = [
            ("/public", "GET", INVALID_CREDS, 200),
            ("/public", "POST", INVALID_CREDS, 401),
            ("/public", "PUT", INVALID_CREDS, 401),
            ("/public", "PATCH", INVALID_CREDS, 401),
            ("/public", "DELETE", INVALID_CREDS, 401),
            ("/public", "OPTIONS", INVALID_CREDS, 200),
        ]
        self.validate(expectations)


class TestDispatchingRedirects(DispatchingTestCase):
    def setUp(self):
        super().setUp()

        self.portal.invokeFactory("Folder", id="private-old")
        self.portal.manage_renameObject("private-old", "private-new")

        self.portal.invokeFactory("Folder", id="public-old")
        public_folder = self.portal["public-old"]
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(public_folder, "publish")
        self.portal.manage_renameObject("public-old", "public-new")

        transaction.commit()

    def test_moved_private_dx_folder_with_creds(self):
        expectations = [
            ("/private-old", "GET", CREDS, 302),
            ("/private-old", "POST", CREDS, 307),
            ("/private-old", "PUT", CREDS, 307),
            ("/private-old", "PATCH", CREDS, 307),
            ("/private-old", "DELETE", CREDS, 307),
            ("/private-old", "OPTIONS", CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/private-old", "GET", CREDS, 200),
            ("/private-old", "POST", CREDS, 200),
            ("/private-old", "PUT", CREDS, 200),
            ("/private-old", "PATCH", CREDS, 200),
            ("/private-old", "DELETE", CREDS, 200),
            ("/private-old", "OPTIONS", CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_private_dx_folder_without_creds(self):
        expectations = [
            ("/private-old", "GET", NO_CREDS, 302),
            ("/private-old", "POST", NO_CREDS, 307),
            ("/private-old", "PUT", NO_CREDS, 307),
            ("/private-old", "PATCH", NO_CREDS, 307),
            ("/private-old", "DELETE", NO_CREDS, 307),
            ("/private-old", "OPTIONS", NO_CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/private-old", "GET", NO_CREDS, 401),
            ("/private-old", "POST", NO_CREDS, 401),
            ("/private-old", "PUT", NO_CREDS, 401),
            ("/private-old", "PATCH", NO_CREDS, 401),
            ("/private-old", "DELETE", NO_CREDS, 401),
            ("/private-old", "OPTIONS", NO_CREDS, 401),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_private_dx_folder_invalid_creds(self):
        expectations = [
            ("/private-old", "GET", INVALID_CREDS, 302),
            ("/private-old", "POST", INVALID_CREDS, 307),
            ("/private-old", "PUT", INVALID_CREDS, 307),
            ("/private-old", "PATCH", INVALID_CREDS, 307),
            ("/private-old", "DELETE", INVALID_CREDS, 307),
            ("/private-old", "OPTIONS", INVALID_CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/private-old", "GET", INVALID_CREDS, 401),
            ("/private-old", "POST", INVALID_CREDS, 401),
            ("/private-old", "PUT", INVALID_CREDS, 401),
            ("/private-old", "PATCH", INVALID_CREDS, 401),
            ("/private-old", "DELETE", INVALID_CREDS, 401),
            ("/private-old", "OPTIONS", INVALID_CREDS, 401),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_with_creds(self):
        expectations = [
            ("/public-old", "GET", CREDS, 302),
            ("/public-old", "POST", CREDS, 307),
            ("/public-old", "PUT", CREDS, 307),
            ("/public-old", "PATCH", CREDS, 307),
            ("/public-old", "DELETE", CREDS, 307),
            ("/public-old", "OPTIONS", CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/public-old", "GET", CREDS, 200),
            ("/public-old", "POST", CREDS, 200),
            ("/public-old", "PUT", CREDS, 200),
            ("/public-old", "PATCH", CREDS, 200),
            ("/public-old", "DELETE", CREDS, 200),
            ("/public-old", "OPTIONS", CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_without_creds(self):
        expectations = [
            ("/public-old", "GET", NO_CREDS, 302),
            ("/public-old", "POST", NO_CREDS, 307),
            ("/public-old", "PUT", NO_CREDS, 307),
            ("/public-old", "PATCH", NO_CREDS, 307),
            ("/public-old", "DELETE", NO_CREDS, 307),
            ("/public-old", "OPTIONS", NO_CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/public-old", "GET", NO_CREDS, 200),
            ("/public-old", "POST", NO_CREDS, 401),
            ("/public-old", "PUT", NO_CREDS, 401),
            ("/public-old", "PATCH", NO_CREDS, 401),
            ("/public-old", "DELETE", NO_CREDS, 401),
            ("/public-old", "OPTIONS", NO_CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_invalid_creds(self):
        expectations = [
            ("/public-old", "GET", INVALID_CREDS, 302),
            ("/public-old", "POST", INVALID_CREDS, 307),
            ("/public-old", "PUT", INVALID_CREDS, 307),
            ("/public-old", "PATCH", INVALID_CREDS, 307),
            ("/public-old", "DELETE", INVALID_CREDS, 307),
            ("/public-old", "OPTIONS", INVALID_CREDS, 307),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ("/public-old", "GET", INVALID_CREDS, 200),
            ("/public-old", "POST", INVALID_CREDS, 401),
            ("/public-old", "PUT", INVALID_CREDS, 401),
            ("/public-old", "PATCH", INVALID_CREDS, 401),
            ("/public-old", "DELETE", INVALID_CREDS, 401),
            ("/public-old", "OPTIONS", INVALID_CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)



from plone.rest.interfaces import IAPIRequest


def __before_publishing_traverse__(self, arg1, arg2=None):
    """Pre-traversal hook that stops traversal to prevent the default view
    to be appended. Appending the default view would break REST calls.
    """
    # XXX hack around a bug(?) in BeforeTraverse.MultiHook
    REQUEST = arg2 or arg1

    if IAPIRequest.providedBy(REQUEST):
        return

    return self._old___before_publishing_traverse__(arg1, arg2)



from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING

import json
import requests
import transaction
import unittest


class TestErrorHandling(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Document", id="document1")
        self.document = self.portal.document1
        self.document_url = self.document.absolute_url()
        self.portal.invokeFactory("Folder", id="folder1")
        self.folder = self.portal.folder1
        self.folder_url = self.folder.absolute_url()
        transaction.commit()

    def test_404_not_found(self):
        response = requests.get(
            self.portal_url + "/non-existing-resource",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Accept: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertTrue(json.loads(response.content))
        self.assertEqual("NotFound", response.json()["type"])
        self.assertEqual(
            "Resource not found: %s" % response.url, response.json()["message"]
        )

    def test_401_unauthorized(self):
        response = requests.get(
            self.document_url, headers={"Accept": "application/json"}
        )
        self.assertEqual(response.status_code, 401)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Accept: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertNotIn(
            "Location", response.headers, "A 401 unauthorized should not redirect."
        )
        self.assertTrue(json.loads(response.content))
        self.assertEqual("Unauthorized", response.json()["type"])

    def test_500_internal_server_error(self):
        response = requests.get(
            self.portal_url + "/500-internal-server-error",
            headers={"Accept": "application/json"},
        )

        self.assertEqual(response.status_code, 500)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Accept: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertTrue(json.loads(response.content))
        self.assertEqual("HTTPError", response.json()["type"])

        self.assertEqual(
            {"type": "HTTPError", "message": "HTTP Error 500: InternalServerError"},
            response.json(),
        )

    def test_500_traceback_only_for_manager_users(self):
        # Normal user
        response = requests.get(
            self.portal_url + "/500-internal-server-error",
            headers={"Accept": "application/json"},
            auth=(TEST_USER_ID, TEST_USER_PASSWORD),
        )
        self.assertNotIn("traceback", response.json())

        # Manager user
        response = requests.get(
            self.portal_url + "/500-internal-server-error",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertIn("traceback", response.json())

        traceback = response.json()["traceback"]
        self.assertIsInstance(traceback, list)
        self.assertRegex(
            traceback[0], r'^File "[^"]*", line \d*, in (publish|transaction_pubevents)'
        )



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.rest.service import Service
from plone.testing import zope
from zope.configuration import xmlconfig


class PloneRestLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.rest

        xmlconfig.file("configure.zcml", plone.rest, context=configurationContext)
        xmlconfig.file("testing.zcml", plone.rest, context=configurationContext)


PLONE_REST_FIXTURE = PloneRestLayer()
PLONE_REST_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_REST_FIXTURE,), name="PloneRestLayer:Integration"
)
PLONE_REST_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_REST_FIXTURE, zope.WSGI_SERVER_FIXTURE),
    name="PloneRestLayer:Functional",
)


class InternalServerErrorService(Service):
    def __call__(self):
        from urllib.error import HTTPError

        raise HTTPError(
            "http://nohost/plone/500-internal-server-error",
            500,
            "InternalServerError",
            {},
            None,
        )



from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING

import requests
import transaction
import unittest


class TestNamedServiceEndpoints(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Document", id="doc1")
        self.document = self.portal["doc1"]
        transaction.commit()

    def test_dexterity_named_get(self):
        response = requests.get(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named get"}, response.json())

    def test_dexterity_named_post(self):
        response = requests.post(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named post"}, response.json())

    def test_dexterity_named_put(self):
        response = requests.put(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named put"}, response.json())

    def test_dexterity_named_patch(self):
        response = requests.patch(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named patch"}, response.json())

    def test_dexterity_named_delete(self):
        response = requests.delete(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named delete"}, response.json())

    def test_dexterity_named_options(self):
        response = requests.options(
            self.document.absolute_url() + "/namedservice",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual({"service": "named options"}, response.json())



from plone.rest.interfaces import ICORSPolicy
from plone.rest.interfaces import IService
from zope.component import queryMultiAdapter
from zope.interface import implementer


@implementer(IService)
class Service:
    def __call__(self):
        policy = queryMultiAdapter((self.context, self.request), ICORSPolicy)
        if policy is not None:
            if self.request._rest_cors_preflight:
                policy.process_preflight_request()
                return
            else:
                policy.process_simple_request()
        else:
            if self.request._rest_cors_preflight:
                return

        return self.render()

    def render(self):
        raise NotImplementedError

    def __getattribute__(self, name):
        # Preflight requests need to be publicly accessible since they don't
        # include credentials
        if name == "__roles__" and self.request._rest_cors_preflight:
            return ["Anonymous"]
        return super().__getattribute__(name)



from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING

import requests
import unittest


class TestSiteRootServiceEndpoints(unittest.TestCase):
    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_siteroot_get(self):
        response = requests.get(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(
            response.status_code,
            200,
            "GET /Plone should return status code 200, not {}".format(
                response.status_code
            ),
        )
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("GET", response.json().get("method"))

    def test_siteroot_post(self):
        response = requests.post(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(
            response.status_code,
            200,
            "POST /Plone should return status code 201, not {}".format(
                response.status_code
            ),
        )
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("POST", response.json().get("method"))

    def test_siteroot_delete(self):
        response = requests.delete(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("DELETE", response.json().get("method"))

    def test_siteroot_put(self):
        response = requests.put(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("PUT", response.json().get("method"))

    def test_siteroot_patch(self):
        response = requests.patch(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("PATCH", response.json().get("method"))

    def test_siteroot_options(self):
        response = requests.options(
            self.portal.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual("plone", response.json().get("id"))
        self.assertEqual("OPTIONS", response.json().get("method"))



from base64 import b64encode
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.rest.service import Service
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from zExceptions import Unauthorized
from zope.event import notify
from ZPublisher.pubevents import PubStart

import unittest


class TestPermissions(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        self.portal.acl_users.userFolderAddUser("admin", "secret", ["Manager"], [])
        login(self.portal, SITE_OWNER_NAME)
        self.portal.invokeFactory("Document", id="doc1")
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(self.portal.doc1, "publish")

    def traverse(self, path="/plone", accept="application/json", method="GET"):
        request = self.layer["request"]
        request.environ["PATH_INFO"] = path
        request.environ["PATH_TRANSLATED"] = path
        request.environ["HTTP_ACCEPT"] = accept
        request.environ["REQUEST_METHOD"] = method
        auth = f"{TEST_USER_NAME}:{TEST_USER_PASSWORD}"
        b64auth = b64encode(auth.encode("utf8"))
        request._auth = "Basic %s" % b64auth.decode("utf8")
        notify(PubStart(request))
        return request.traverse(path)

    def test_traverse_with_insufficient_permission_raises_unauthorized(self):
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        with self.assertRaises(Unauthorized):
            self.traverse("/plone/doc1", method="PUT")

    def test_traverse_with_sufficient_permission_returns_service(self):
        setRoles(self.portal, TEST_USER_ID, ["Editor"])
        obj = self.traverse("/plone/doc1", method="PUT")
        self.assertTrue(isinstance(obj, Service), "Not a service")



from plone.app.testing import popGlobalRegistry
from plone.app.testing import pushGlobalRegistry
from plone.rest.cors import CORSPolicy
from plone.rest.interfaces import ICORSPolicy
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from zExceptions import Unauthorized
from zope.component import provideAdapter
from zope.event import notify
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from ZPublisher.pubevents import PubStart

import unittest


class TestCORSPolicy(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.response = self.request.response

    def get_policy(self, origin=None, method=None, headers=None):
        request = self.request
        if origin:
            request.environ["HTTP_ORIGIN"] = origin
        if method:
            request.environ["HTTP_ACCESS_CONTROL_REQUEST_METHOD"] = method
        if headers:
            request.environ["HTTP_ACCESS_CONTROL_REQUEST_HEADERS"] = headers
        policy = CORSPolicy(None, request)
        policy.allow_origin = ["*"]
        policy.allow_credentials = False
        policy.allow_methods = ["DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT"]
        policy.allow_headers = []
        policy.expose_headers = []
        policy.max_age = None
        return policy

    def test_simple_cors_without_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy()
        self.assertFalse(policy.process_simple_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_simple_cors_with_wrong_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://wrong.net")
        policy.allow_origin = ["http://example.net"]
        self.assertFalse(policy.process_simple_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_simple_cors_adds_wildcard_allow_origin(self):
        policy = self.get_policy(origin="http://example.net")
        self.assertTrue(policy.process_simple_request())
        self.assertEqual("*", self.response.getHeader("Access-Control-Allow-Origin"))

    def test_simple_cors_adds_matching_allow_origin(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ["http://example.net"]
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            "http://example.net", self.response.getHeader("Access-Control-Allow-Origin")
        )

    def test_simple_cors_adds_allow_credentials(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ["http://example.net"]
        policy.allow_credentials = True
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            "true", self.response.getHeader("Access-Control-Allow-Credentials")
        )

    def test_simple_cors_adds_origin_when_supporting_credentials(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_credentials = True
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            "http://example.net", self.response.getHeader("Access-Control-Allow-Origin")
        )
        self.assertEqual(
            "true", self.response.getHeader("Access-Control-Allow-Credentials")
        )

    def test_simple_cors_adds_vary_when_allowing_multiple_origins(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ["http://some.host", "http://example.net"]
        self.assertTrue(policy.process_simple_request())
        self.assertEqual("Origin", self.response.getHeader("Vary"))

    def test_simple_cors_adds_exposed_headers(self):
        policy = self.get_policy(origin="http://example.net")
        policy.expose_headers = ["Content-Length", "X-My-Header"]
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            "Content-Length, X-My-Header",
            self.response.getHeader("Access-Control-Expose-Headers"),
        )

    def test_preflight_cors_without_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy()
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_preflight_cors_with_wrong_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://wrong.net")
        policy.allow_origin = ["http://example.net"]
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_preflight_cors_with_wrong_method_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://example.net", method="LOCK")
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_preflight_cors_with_wrong_header_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://example.net", headers="X-Secret")
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(None, self.response.getHeader("Access-Control-Allow-Origin"))

    def test_preflight_cors_adds_wildcard_allow_origin(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("*", self.response.getHeader("Access-Control-Allow-Origin"))

    def test_preflight_cors_adds_matching_allow_origin(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_origin = ["http://example.net"]
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            "http://example.net", self.response.getHeader("Access-Control-Allow-Origin")
        )

    def test_preflight_cors_adds_allow_credentials(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_origin = ["http://example.net"]
        policy.allow_credentials = True
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            "true", self.response.getHeader("Access-Control-Allow-Credentials")
        )

    def test_preflight_cors_adds_origin_when_supporting_credentials(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_credentials = True
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            "http://example.net", self.response.getHeader("Access-Control-Allow-Origin")
        )
        self.assertEqual(
            "true", self.response.getHeader("Access-Control-Allow-Credentials")
        )

    def test_preflight_cors_adds_vary_when_allowing_multiple_origins(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_origin = ["http://some.host", "http://example.net"]
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("Origin", self.response.getHeader("Vary"))

    def test_preflight_cors_adds_max_age(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.max_age = "3600"
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("3600", self.response.getHeader("Access-Control-Max-Age"))

    def test_preflight_cors_adds_allow_methods(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("GET", self.response.getHeader("Access-Control-Allow-Methods"))

    def test_preflight_cors_adds_allow_methods_if_unrestriced(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_methods = None
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("GET", self.response.getHeader("Access-Control-Allow-Methods"))

    def test_preflight_cors_adds_allow_headers(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        policy.allow_headers = ["X-Allowed"]
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            "X-Allowed", self.response.getHeader("Access-Control-Allow-Headers")
        )

    def test_preflight_cors_sets_content_length_zero(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual("0", self.response.getHeader("Content-Length"))

    def test_preflight_cors_sets_status_code_200(self):
        policy = self.get_policy(origin="http://example.net", method="GET")
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(200, self.response.getStatus())


class TestCORS(unittest.TestCase):
    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

    def traverse(self, path="/plone", method="GET", headers={}):
        request = self.layer["request"]
        request.environ["PATH_INFO"] = path
        request.environ["PATH_TRANSLATED"] = path
        request.environ["REQUEST_METHOD"] = method
        request.environ["HTTP_ORIGIN"] = "http://example.net"
        request.environ.update(headers)
        notify(PubStart(request))
        return request.traverse(path)

    def test_preflight_cors_is_accessible_anonymously(self):
        headers = {}
        headers["HTTP_ACCESS_CONTROL_REQUEST_METHOD"] = "POST"
        try:
            self.traverse(method="OPTIONS", headers=headers)
        except Unauthorized:
            self.fail("Service not accessible for preflight.")

    def test_simple_cors_gets_processed(self):
        headers = {}
        headers["HTTP_ACCEPT"] = "application/json"
        obj = self.traverse(method="GET", headers=headers)
        obj()
        self.assertEqual(
            "*", self.request.response.getHeader("Access-Control-Allow-Origin")
        )

    def test_preflight_request_without_cors_policy_doesnt_render_service(self):
        # "Unregister" the current CORS policy
        class NoCORSPolicy:
            def __new__(cls, context, request):
                return None

        pushGlobalRegistry(self.portal)
        provideAdapter(NoCORSPolicy, (Interface, IDefaultBrowserLayer), ICORSPolicy)

        headers = {"HTTP_ACCESS_CONTROL_REQUEST_METHOD": "GET", "HTTP_ACCEPT": "*/*"}
        service = self.traverse(method="OPTIONS", headers=headers)
        self.assertTrue(service() is None, "Should return None")

        popGlobalRegistry(self.portal)



from setuptools import find_packages
from setuptools import setup

import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


version = "5.0.0"

long_description = read("README.rst") + "\n\n" + read("CHANGES.rst") + "\n\n"


setup(
    name="plone.rest",
    version=version,
    description="Plone support for HTTP verbs.",
    long_description=long_description,
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: 6.1",
        "Framework :: Plone :: Core",
        "Framework :: Zope2",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="rest http",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/plone.rest/",
    license="GPL version 2",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["plone"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    extras_require=dict(
        test=[
            "plone.app.testing[robot]>=4.2.2",
            "requests",
            "BTrees",
            "plone.app.contenttypes[test]",
            "plone.app.redirector",
            "plone.app.textfield",
            "plone.namedfile",
            "plone.testing",
            "z3c.relationfield",
            "zope.intid",
        ]
    ),
    install_requires=[
        "setuptools",
        "collective.monkeypatcher",
        "Products.CMFCore",
        "Zope",
        "plone.memoize",
        "zope.browserpage",
    ],
    entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
)



from plone.rest.cors import lookup_preflight_service_id
from plone.rest.interfaces import IAPIRequest
from plone.rest.negotiation import lookup_service_id
from zope.interface import alsoProvides


def mark_as_api_request(request, accept):
    """Mark a request as IAPIRequest if there's a service registered for the
    actual request method and Accept header.
    """
    method = request.get("REQUEST_METHOD", "GET")
    if method == "OPTIONS" and request.getHeader("Origin", False):
        preflighted_method = request.getHeader("Access-Control-Request-Method", None)
        service_id = lookup_preflight_service_id(preflighted_method)
        request._rest_cors_preflight = True
    else:
        service_id = lookup_service_id(method, accept)
        request._rest_cors_preflight = False

    if service_id is not None:
        alsoProvides(request, IAPIRequest)
        request._rest_service_id = service_id

        # Flag as non-WebDAV request in order to avoid special treatment
        # in ZPublisher.BaseRequest.traverse().
        request.maybe_webdav_client = 0


def subscriber_mark_as_api_request(event):
    """Subscriber to mark a request as IAPIRequest (see mark_as_api_request)"""
    mark_as_api_request(
        event.request,
        event.request.getHeader("Accept", "text/html"),
    )
